@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 2002, 2003, 2004, 2005, 2006 by                               */
+/*  Copyright 2002, 2003, 2004, 2005, 2006, 2009, 2010 by                   */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*  ftbench: bench some common FreeType call paths                          */
@@ -21,7 +21,7 @@
 #include FT_CACHE_IMAGE_H
 #include FT_CACHE_SMALL_BITMAPS_H
 #include FT_SYNTHESIS_H
-
+#include FT_ADVANCES_H
 
 #ifdef UNIX
 #include <sys/time.h>
@@ -74,6 +74,7 @@ FTC_ImageTypeRec  font_type;
 
 enum {
   FT_BENCH_LOAD_GLYPH,
+  FT_BENCH_LOAD_ADVANCES,
   FT_BENCH_RENDER,
   FT_BENCH_GET_GLYPH,
   FT_BENCH_GET_CBOX,
@@ -86,6 +87,7 @@ enum {
 
 const char* bench_desc[] = {
   "Load a glyph",
+  "Load advance widths",
   "Render a glyph",
   "Get FT_Glyph",
   "Get glyph cbox",
@@ -224,6 +226,31 @@ test_load( btimer_t*  timer,
 }
 
 
+int
+test_load_advances( btimer_t*  timer,
+                    FT_Face    face,
+                    void*      user_data )
+{
+  int        done = 0;
+  FT_Fixed*  advances;
+  FT_ULong   flags = *((FT_ULong*)user_data);
+
+
+  advances = (FT_Fixed *)calloc( sizeof ( FT_Fixed ), face->num_glyphs );
+
+  TIMER_START( timer );
+
+  FT_Get_Advances( face, 0, face->num_glyphs, flags, advances );
+  done += face->num_glyphs;
+
+  TIMER_STOP( timer );
+
+  free( advances );
+
+  return done;
+}
+
+
 int
 test_render( btimer_t*  timer,
              FT_Face    face,
@@ -735,7 +762,8 @@ main(int argc,
 
   for ( i = 0; i < N_FT_BENCH; i++ )
   {
-    btest_t  test;
+    btest_t   test;
+    FT_ULong  flags;
 
 
     if ( !TEST( 'a' + i ) )
@@ -766,6 +794,19 @@ main(int argc,
         benchmark( face, &test, max_iter, max_time );
       }
       break;
+    case FT_BENCH_LOAD_ADVANCES:
+      test.user_data = &flags;
+
+      test.title = "Load_Advances (Normal)";
+      test.bench = test_load_advances;
+      flags      = FT_LOAD_DEFAULT;
+      benchmark( face, &test, max_iter, max_time );
+
+      test.title  = "Load_Advances (Fast)";
+      test.bench  = test_load_advances;
+      flags       = FT_LOAD_TARGET_LIGHT;
+      benchmark( face, &test, max_iter, max_time );
+      break;
     case FT_BENCH_RENDER:
       test.title = "Render";
       test.bench = test_render;
