@@ -12,6 +12,7 @@
 #include FT_FREETYPE_H
 #include FT_SFNT_NAMES_H
 #include FT_TRUETYPE_IDS_H
+#include FT_TRUETYPE_TABLES_H
 
   /* the following header shouldn't be used in normal programs */
 #include FT_INTERNAL_DEBUG_H
@@ -84,7 +85,7 @@
   Print_Name( FT_Face  face )
   {
     const char*  ps_name;
-    
+
 
     printf( "font name entries\n" );
 
@@ -92,11 +93,11 @@
 
     printf( "   family:     %s\n", face->family_name );
     printf( "   style:      %s\n", face->style_name );
-    
+
     ps_name = FT_Get_Postscript_Name( face );
     if ( ps_name == NULL )
       ps_name = "UNAVAILABLE";
-    
+
     printf( "   postscript: %s\n", ps_name );
   }
 
@@ -104,7 +105,7 @@
   void
   Print_Type( FT_Face  face )
   {
-    FT_ModuleRec*  module;
+    FT_Module  module;
 
 
     printf( "font type entries\n" );
@@ -145,7 +146,7 @@
 
     printf( "   glyph names:     %s\n",
             FT_HAS_GLYPH_NAMES( face ) ? (char *)"yes" : (char *)"no" );
-    
+
     if ( FT_IS_SCALABLE( face ) )
     {
       printf( "   EM size:         %d\n", face->units_per_EM );
@@ -508,10 +509,11 @@
 
     for( i = 0; i < face->num_charmaps; i++ )
     {
-      printf( "   %d: platform %d, encoding %d",
+      printf( "   %d: platform %d, encoding %d, language %d",
               i,
               face->charmaps[i]->platform_id,
-              face->charmaps[i]->encoding_id );
+              face->charmaps[i]->encoding_id,
+              (FT_UInt)FT_Get_CMap_Language_ID( face->charmaps[i] ) );
       if ( i == active )
         printf( " (active)" );
       printf ( "\n" );
@@ -607,7 +609,7 @@
     if ( debug )
     {
       char  temp[32];
-      
+
 
       sprintf( temp, "any=%d", trace_level );
       setenv( "FT2_DEBUG", temp );
